PROGRAM TIMESTMP;
(*

        Copyright   This program, including this source code,
        Notice      is copyright (c) 1996 by Radiodetection Corp.
                    All rights reserved.


        Does:       DOS filter to prepend DATE and TIME.
        Language:   Turbo Pascal
        Programmer: Russ Bellew
        Created:    2 Jan 96
        Requires:
          Compiler:
            Borland's Turbo Pascal ver 6.0
         Runtime Environment:
            MS-DOS;


        Revisions:

        v1.1 - Fixed Sun 15 Sep 96 reported as Monday bug.
               Removed dependence on OpString.tpu.  RB, 18-Sep-96

*)

{$DEFINE production}

{$IFDEF production} {$D-} {$R-} {$L-} {$S-} {$ENDIF}


USES Dos,
     Crt;

CONST
  ProgName   = 'TIMESTMP';
  Version    = '1.1';
  Date       = '18 Sep, 1996';
  Copyright  = 'Copyright (c) 1996 by Radiodetection Corp. & Russ Bellew';
  moNameLen  = 3;                         { Length of each month name }
  monthNames = 'JanFebMarAprMayJunJulAugSepOctNovDec';
  dayNameLen = 3;
  dayNames   = 'SunMonTueWedThuFriSat';
  LF         = 10;                        { ASCII characters }
  CR         = 13;

TYPE
  PmType = (AM, PM);
  DisplayType = text;

VAR
  TwelveHourWanted,
  SecsWanted,
  DateOnlyWanted,
  TimeOnlyWanted,
  DayNameWanted,
  CenturyWanted,
  LineFeedWanted,
  AppendWanted,
  Quiet                 : boolean;

  ch                    : Char;

  TimeValStr            : string[10];
  DateValStr            : string[16];

  h, m, secs, hund,     { Units of time }
  year, monthNo,
  day, dow              : Word;

  AmOrPm                : PmType;
  PmStr                 : string[2];
  DayStr                : String[2];
  WeSucceeded           : Boolean;
  ErrorCode             : Byte;
  CrtDisplay            : DisplayType;



function IntToStr(i: Longint): string;
{ Convert any Integer type to a string }
var
  s: string[11];
begin
  Str(i, s);
  IntToStr := s;
end;




function LeadingZero(w : Word) : String;
var
  s : String;
begin
  Str(w:0,s);
  if Length(s) = 1 then
    s := '0' + s;
  LeadingZero := s;
end;


function StUpCase(s : string) : string;
var i : Integer;
begin
  for i := 1 to Length(s) do
    s[i] := UpCase(s[i]);
  StUpcase := s;
end;




procedure GetParameters;
{ get command line parameters used to start this program }
var
  i    : byte;
  s    : string;
  Numeric : boolean;
begin
  i                  := 0;
  TwelveHourWanted   := False;
  SecsWanted         := False;
  DateOnlyWanted     := False;
  TimeOnlyWanted     := False;
  DayNameWanted      := False;
  CenturyWanted      := True;
  LineFeedWanted     := False;
  Quiet              := False;
  Numeric            := False;

  for i := 1 to ParamCount do begin
    s := StUpCase(ParamStr(i));
    if (s[1] = '/') or (s[1] = '-') then Delete(s, 1, 1);

    case s[1] of
      '1'     : TwelveHourWanted := True;
      'A'     : AppendWanted     := True;
      'L'     : LineFeedWanted   := True;
      'D'     : DateOnlyWanted := True;
      'T'     : TimeOnlyWanted := True;
      'S'     : SecsWanted := True;
      'Q'     : Quiet := True;
      'N'     : DayNameWanted  := True;
      'Y'     : CenturyWanted  := False;
      'H','?' : begin
                  Crt.ClrScr;
                  WriteLn(ProgName,'   ',Version, '  ',Date);
                  WriteLn(Copyright);
                  WriteLn('This program is a DOS filter that prepends current DOS date and time to');
                  WriteLn('DOS''s standard input.');
                  WriteLn;
                  WriteLn('   Option            What it does                             Default');
                  WriteLn('   ------            -------------------------                -------');
                  WriteLn('   /12               Use 12 hour clock                        24 hour');
                  WriteLn('   /S                Use seconds = yes                        no');
                  WriteLn('   /D                Date only (omit time)                    Date + Time');
                  WriteLn('   /T                Time only (omit date)                    Date + Time');
                  WriteLn('   /N                Print day name                           no day name');
                  WriteLn('   /Y                Omit century from year  (YY only)        YYYY');
                  WriteLn('   /L                Timestamp on its own line = yes          no');
                  WriteLn('   /A                Append timestamp                         pre-pend');
                  WriteLn('   /?                Display this help screen');
                  WriteLn;
                  WriteLn('Examples:   ECHO - Started log file.|', ProgName, ' /12 /S>>LOG.FIL');
                  WriteLn('            TYPE ASCIIFIL.TXT|', ProgName, ' /12 /L /N>ASCIIFIL.TXT');
                  WriteLn('Use this program as you like, at no cost.              All Rights Reserved.');
                  WriteLn('                 Radiodetection Corp.   Phone: 201 848-8070');
                  WriteLn(' "Manufacturer of the world''s best buried cable and pipe locating equipment"');
                  WriteLn('                   35 Whitney Road, Mahwah, NJ  07430 USA');
                  Halt(0);
                end; { 'H','?' : begin }
    end; { case s[1] of }

  end; { for i = 1 to paramcount }
end;



Begin { program }
  ErrorCode := 0;
  Crt.DirectVideo  := True;

  Assign(CrtDisplay,'');
  Rewrite(CrtDisplay);


  { Get input from standard input. Open it as a text file, and
    later we'll read it char by char. }
  Close(Input);
  Assign(Input,'');
  Reset(Input);


  GetParameters;

  { Standard output }
  Close(Output);
  Assign(Output,'');
  Rewrite(Output);


  Dos.GetTime(h,m,secs,hund);
  if TwelveHourWanted then
    begin
      if h > 11 then
        PmStr := 'p'
      else
        PmStr := 'a';
      if h > 12 then h := h - 12;
    end else PmStr := '';

   Dos.GetDate(year,monthNo,day,dow);


  if DateOnlyWanted then
    TimeValStr := ''
  else
    begin
      if TwelveHourWanted then
          str(h:2, TimeValStr)             { Leading space }
      else
          TimeValStr := LeadingZero(h);
      TimeValStr := TimeValStr + ':' + LeadingZero(m);

      if SecsWanted then TimeValStr := TimeValStr + ':' + LeadingZero(secs);
      TimeValStr := TimeValStr + PmStr;
    end;

  DateValStr := '';
  DayStr := '';
  if not TimeOnlyWanted then
    begin
      if DayNameWanted then DateValStr := Copy(dayNames,
                         (dow * dayNameLen) + 1,
                         dayNameLen) + ' ';
      str(day:2, DayStr);   { leading space }
      DateValStr := DateValStr + DayStr
                    + '-' +
                  Copy(monthNames, (monthNo * moNameLen)
                  - (moNameLen - 1), moNameLen) + '-';
      if CenturyWanted then
         DateValStr := DateValStr + IntToStr(year) + ' '
      else
         DateValStr := DateValStr + Copy(IntToStr(year), 3, 2) + ' ';
    end;

  if AppendWanted then
    begin
        while not Eof(Input) do                 { Re-direct input }
          begin
            Read(Input, ch);
            Write(Output, ch);
          end;
        if LineFeedWanted then WriteLn(Output); { Write our stuff }
        Write(Output, ' ' + DateValStr + TimeValStr);
    end
  else
    begin
        Write(Output, DateValStr + TimeValStr); { Write our stuff }
        if LineFeedWanted then
           WriteLn(Output)
        else Write(Output, ' ');
        while not Eof(Input) do                 { Re-direct input }
          begin
            Read(Input, ch);
            Write(Output, ch);
          end;
    end;
  Close(Output);
  Close(Input);


  Halt(ErrorCode);


end. { program }
